#include "graph_blocks.h"
#include <stdlib.h>

// See graph_blocks.h
const struct graph_node_type* blockDefs[MAX_BLOCK_TYPES] = 
{
    &node_const_type,
    &node_add_type,
    &node_mult_type,
    &node_gain_type,
    &node_accum_type,
    &node_bounds_type,
    &node_mixer_type,
    &node_pid_type
};


int graph_add_defined_block(struct computation_graph* graph, int type_id, const char* name) {
    // Verify block type is valid
    if (type_id >= MAX_BLOCK_TYPES) {
        return -1;
    }
    const struct graph_node_type *block_type = blockDefs[type_id];
    void* state = NULL;
    // Allocate the state struct for this node, if necessary
    if (block_type->state_size) {
        state = malloc(block_type->state_size);
        if (!state) {return -1;} // Check for malloc failure
    }

    // Use the computation graph implementation's add node function
    return graph_add_node(graph, name, block_type, state);
}