#include "hw_impl_unix.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <pthread.h>
#include <unistd.h>

void * output_cache();

static char *output_pwms[4];
static unsigned long cache[4];
pthread_t worker;

static int zero = 0;
static int one = 1;
static int two = 2;
static int three = 3;

int unix_pwm_output_reset(struct PWMOutputDriver *self) {
  output_pwms[0] = VIRT_QUAD_FIFOS_DIR "/pwm-output-motor1";
  output_pwms[1] = VIRT_QUAD_FIFOS_DIR "/pwm-output-motor2";
  output_pwms[2] = VIRT_QUAD_FIFOS_DIR "/pwm-output-motor3";
  output_pwms[3] = VIRT_QUAD_FIFOS_DIR "/pwm-output-motor4";

  mkdir(VIRT_QUAD_FIFOS_DIR, 0777);

  // Start up worker thread whose job is to update the caches
  pthread_create(&worker, 0, output_cache, &zero);
  pthread_create(&worker, 0, output_cache, &one);
  pthread_create(&worker, 0, output_cache, &two);
  pthread_create(&worker, 0, output_cache, &three);

  return 0;
}

int unix_pwm_output_write(struct PWMOutputDriver *self,
                          unsigned int channel,
                          unsigned long pulse_width_us) {
  if (cache[channel] != pulse_width_us) {
    printf("%s: %ld\n", output_pwms[channel], pulse_width_us);
  }
  cache[channel] = pulse_width_us;
  return 0;
}

void * output_cache(void *arg) {
  int *output_index = arg;
  char buff[16];
  int i = *output_index;

  // Setup FIFO
  unlink(output_pwms[i]);
  mkfifo(output_pwms[i], 0666);

  // Block while waiting for someone to listen
  while (1) {
    int fifo = open(output_pwms[i], O_WRONLY);
    sprintf(buff, "%ld\n", cache[i]);
    write(fifo, buff, strlen(buff));
    close(fifo);
    usleep(500); // don't spam the reader
  }
  return NULL;
}
